function [] = find_F0_contour_halcyon(audiofolname,filename)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIND_F0_CONTOUR_HALCYON.M finds the fundamental frequency contour using Halcyon, the
% pitch strength contour using A-SWIPE', and the whole sentence SNR
%
% Author: Jenny Vojtech (jmvo@bu.edu)
%         Stepp Lab, Boston University, MA
% Date:   10/28/2018

%Edited By: Matti Groll (mgroll@bu.edu)
%           Stepp Lab, Boston Unviersity, MA
%Date:      3/25/2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%Initialize global variables
global min_pitch_set;   %minimum F0 (Hz)
global max_pitch_set;   %maximum F0 (Hz)
global perform_fol;     %output folder

%%%Declare folders of interest
aRFF.subjectid = filename(1:end-4);
folname = [perform_fol aRFF.subjectid '\var\'];
f0cfile = [folname 'f0c_param.mat']; % F0 contour variables

if ~exist(folname,'dir')
    mkdir(folname);
end

%%%If f0cfile already exist, skip this step (return to main_RFF_aswipep.m)
% if exist(f0cfile,'file')
%     return;
% end
%%%Read in audio file
%in = speech signal (V)
%fs = sampling frequency (Hz)
%t  = t vector for speech signal (s)
%N  = number of samples
[in, fs] = audioread([audiofolname,filename]);
t        = linspace(0,length(in)/fs,length(in));
N        = length(t);

%%%If there are multiple channels, only read in the first channel
if size(in,2) > 1
    in = in(:,1);
end
in = (in)./(0.9*max(abs(in))); % Peak-normalize the audio file

%%%Collect signal and noise
%v_prob is high there is likely to be a vowel (energy in 0 - 3.4 kHz)
[B,A] = butter(5,(3400)/(fs/2),'low');
lowy = filtfilt(B,A,in);
sm_lowy = smooth(abs(lowy),fs/20);
v_prob = (sm_lowy - mean(sm_lowy))./std(sm_lowy);
v_prob = v_prob - min(v_prob);

%f_prob is high when there is likely to be a fricative (energy in
%3.4 kHz to 8 kHz)
[B,A] = butter(5,[(3400)/(fs/2) (8000)/(fs/2)]);
highy = filtfilt(B,A,in);
sm_highy = smooth(abs(highy),fs/20);
fric_like = smooth(sm_highy./sm_lowy,fs/20);
f_prob = (fric_like - mean(fric_like))./std(fric_like);
f_prob = f_prob - min(f_prob);

fv_diff_prob_temp = f_prob - v_prob;
fv_diff_prob_temp = smooth(fv_diff_prob_temp,fs/20);
xin = rms(in(fv_diff_prob_temp<0));

start_noise = in(1:round(fs*0.05));
end_noise = in(end-round(fs*0.05):end);
if rms(start_noise) > 2*rms(end_noise)
    noise_vector = end_noise;
elseif rms(end_noise) > 2*rms(start_noise)
    noise_vector = start_noise;
else
    noise_vector = [start_noise ; end_noise];
end
xnoise = rms(noise_vector);

%%%Calculate SNR
wSNR = 20*log10(xin/xnoise); % wSNR, in dB

%%%Define A-SWIPE' parameters
minF0    = min_pitch_set; %minimum F0 (Hz)
maxF0    = max_pitch_set; %maximum F0 (Hz)
dt       = 0.001;         %time interval for F0 estimation
woverlap = 0.6;           %Hann window overlap (0-1)
STHR     = 0;             %pitch strength threshold
dERBs    = 1/32;          %spectrum step size
dlog2p   = 1/32;          %resolution in steps per octave

%%%Find pitch strength contour using A-SWIPE'
[P,T,S] = aswipep(in,fs,[minF0,maxF0],dt,dlog2p,dERBs,woverlap,STHR);
savePSc = interp1(T,S,t,'linear'); %pitch strength (PS) contour
savePSc = savePSc - min(savePSc);
savePSc(isnan(savePSc)) = 0.01;    %remove NaN from PS contour
ps = mean(savePSc((fv_diff_prob_temp<0)));

%%%Find fundamental frequency contour using Halcyon
param.min_f0 = min_pitch_set;   % Minimum F0 candidate
param.max_f0 = max_pitch_set;   % Maximum F0 candidate
param.N_cndt = 100;             % # F0 candidates
param.L = 4;                    % # cycles within the time window
param.K = 8;                    % # harmonics used = # of filters in filter bank
param.V = 1;                    % # shifts for instant phase estimates (>0)
param.fstep  = 8;               % Frame input step (controls time resolution)
param.dpstep = 21;              % Dynamic prog. step (# samples to jump)
[ph, th] = halcyon(in, fs, param);
saveF0c = interp1(th', ph, t', 'linear'); % F0 contour
saveF0c(isnan(saveF0c)) = 0.01;

aRFF.SNR = wSNR;
aRFF.ps = ps;
aRFF.F0c = saveF0c;
aRFF.PSc = savePSc;

save(f0cfile,'saveF0c','aRFF') %save resultant vectors

end